﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using Autodesk.Navisworks.Api;
using Autodesk.Navisworks.Api.DocumentParts;
 
namespace NW2013Demo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnGrid_Click(object sender, EventArgs e)
        {     
            GridsOptions oGOpt = Autodesk.Navisworks.Api.Application.Options.Grids;
            ColorDialog oColorDlg = new ColorDialog();
            if(oColorDlg.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                Autodesk.Navisworks.Api.Color oColor = 
                    new Autodesk.Navisworks.Api.Color( oColorDlg.Color.R,
                        oColorDlg.Color.G,
                         oColorDlg.Color.B);

                oGOpt.AboveColor  = oColor;
                oGOpt.BelowColor = oColor; 
            }
        }

        private void treeView1_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
 
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            Document oDoc = Autodesk.Navisworks.Api.Application.MainDocument;
            DocumentGrids oGrids = oDoc.Grids;

            foreach (GridSystem oGS in oGrids.Systems)
            {
                TreeNode oNode = treeView1.Nodes.Add(oGS.DisplayName);
                oNode.Tag = oGS;
            }

            foreach (string oStr in Enum.GetNames(typeof(GridsRenderMode)))
                listBox1.Items.Add(oStr);

             
            foreach (Comment oStr in oDoc.CurrentComments.Value)
                listBox2.Items.Add(oStr);
        }

    

        private void treeView1_AfterSelect(object sender, TreeViewEventArgs e)
        {
            try
            {
                TreeNode oSelNode = treeView1.SelectedNode;
                if (oSelNode != null)
                {
                    object oTag = oSelNode.Tag;
                    if (oTag != null)
                    {
                        GridSystem oGS = (GridSystem)oTag;
                        if (oGS != null)
                        {
                            Document oDoc = Autodesk.Navisworks.Api.Application.MainDocument;
                            oDoc.Grids.ActiveSystem = oGS;                            
                            oDoc.ActiveView.FocusOnCurrentSelection();
            
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show("error!");
            }
        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {
            GridsOptions oGOpt = Autodesk.Navisworks.Api.Application.Options.Grids;
            if (checkBox1.Checked)
            {
                oGOpt.Enabled = true;
            }

            else
            {
                oGOpt.Enabled = false;
            }
        }

        private void listBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            Document oDoc = Autodesk.Navisworks.Api.Application.MainDocument;
            if(oDoc.Grids.Systems.Count != 0)
            {
                GridsRenderMode value;
                Array values = Enum.GetValues(typeof(GridsRenderMode));
                value = (GridsRenderMode)values.GetValue(Convert.ToInt32(listBox1.SelectedIndex));
                Autodesk.Navisworks.Api.Application.ActiveDocument.Grids.RenderMode = value;
            }
             
        }

        private void btnComments_Click(object sender, EventArgs e)
        {
            Document oDoc = Autodesk.Navisworks.Api.Application.MainDocument;
            try
            {
                CommentCollection oCopyCommentsColl = new CommentCollection(oDoc.CurrentComments.Value);
                oCopyCommentsColl.Add(new Comment("newComments", CommentStatus.Approved)); ;

                oDoc.CurrentComments.CopyFrom(oCopyCommentsColl);
            }
            catch (Exception ex)
            {

            }
            
            listBox2.Items.Clear();
            foreach (Comment oStr in oDoc.CurrentComments.Value)
                listBox2.Items.Add(oStr);
        }


        private void recurse(SavedItem oFolder, TreeNode parentNode)
        {
            foreach (SavedViewpoint oSVP in ((GroupItem)oFolder).Children)
            {
                TreeNode oNode = parentNode.Nodes.Add(oSVP.DisplayName);
                oNode.Tag = oSVP;

                if (oSVP.IsGroup)
                    recurse(oSVP, oNode);
            }
        }

        private void refreshSavedPt()
        {
            treeView_saveviewpoints.Nodes.Clear();
            Document oDoc = Autodesk.Navisworks.Api.Application.ActiveDocument;

            foreach (SavedItem oSVP in oDoc.SavedViewpoints.Value)
            {
                TreeNode oNode = treeView_saveviewpoints.Nodes.Add(oSVP.DisplayName);
                oNode.Tag = oSVP;

                if (oSVP.IsGroup)
                    recurse(oSVP, oNode);
            } 
        }
        private void btnRefresh_Click(object sender, EventArgs e)
        {
            refreshSavedPt();
 
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            Document oDoc = Autodesk.Navisworks.Api.Application.ActiveDocument;

            Transaction oTran = oDoc.BeginTransaction("AddNewViewPoints");
            try
            {

                // 创建第一个保存视点，使用当前视点
                SavedViewpoint oNewViewPt1 = new SavedViewpoint(oDoc.CurrentViewpoint.ToViewpoint());
                oNewViewPt1.DisplayName = "MyViewPt1";

                // 为第二个保存视点创建视点
                Viewpoint oNewVP = new Viewpoint();
                oNewVP = oDoc.CurrentViewpoint.ToViewpoint().CreateCopy();
                Point3D oNewPos = new Point3D(oNewVP.Position.X + 10,
                                                oNewVP.Position.Y,
                                                  oNewVP.Position.Z);
                oNewVP.Position = oNewPos;

                // 创建第二个保存视点
                SavedViewpoint oNewViewPt2 = new SavedViewpoint(oNewVP);
                oNewViewPt2.DisplayName = "MyViewPt2";


                SavedViewpoint oNewViewPt3 = new SavedViewpoint(oDoc.CurrentViewpoint.ToViewpoint());
                oNewViewPt3.DisplayName = "MyViewPt3";

                // 创建一个保存视点组
                FolderItem oNewViewPtFolder1 = new FolderItem();
                oNewViewPtFolder1.DisplayName = "MyViewPtFolder1";
                // 将第一个新的保存视点放到该组
                oNewViewPtFolder1.Children.Add(oNewViewPt1);
                oNewViewPtFolder1.Children.Add(oNewViewPt2);


                FolderItem oNewViewPtFolder2 = new FolderItem();
                oNewViewPtFolder2.DisplayName = "MyViewPtFolder2";
                oNewViewPtFolder2.Children.Add(oNewViewPt3);

                // 添加视点组
                oDoc.SavedViewpoints.AddCopy(oNewViewPtFolder1);
                // 添加视点
                oDoc.SavedViewpoints.AddCopy(oNewViewPtFolder2);

                oTran.Commit();

                refreshSavedPt();
            }
            catch (Exception ex)
            {
                oTran.Dispose();
            }   

        }

        private void MoveX(double val)
        {
            Document oDoc = Autodesk.Navisworks.Api.Application.ActiveDocument;
            Autodesk.Navisworks.Api.Transaction oTran = oDoc.BeginTransaction("Move Camera");
            try
            {
                Viewpoint oCurrVCopy = oDoc.CurrentViewpoint.CreateCopy();
                Point3D newPos = new Point3D(oCurrVCopy.Position.X + val,
                                             oCurrVCopy.Position.Y,
                                             oCurrVCopy.Position.Z);

                oCurrVCopy.Position = newPos;
                oDoc.CurrentViewpoint.CopyFrom(oCurrVCopy);
                oTran.Commit();
            }
            catch (Exception ex)
            {
                oTran.Dispose();
            }
        }

        private void buttonRight_Click(object sender, EventArgs e)
        {
            MoveX(double.Parse(textStep.Text));
        }

        private void buttonLeft_Click(object sender, EventArgs e)
        {
            MoveX(-double.Parse(textStep.Text));
        }

        private void RotateCamera(double angle)
        {
            Document oDoc = Autodesk.Navisworks.Api.Application.ActiveDocument;

            Autodesk.Navisworks.Api.Transaction oTran = oDoc.BeginTransaction("Rotate Camera");

            Viewpoint oCurrV = oDoc.CurrentViewpoint;


            AxisAndAngleResult oAA = oCurrV.Rotation.ToAxisAndAngle();

            UnitVector3D oAxis = oAA.Axis;
            double oldangle = oAA.Angle;


            UnitVector3D odeltaA = new UnitVector3D(Double.Parse(textRotX.Text), Double.Parse(textRotY.Text), Double.Parse(textRotZ.Text));

            Rotation3D delta = new Rotation3D(odeltaA, angle);

          
            try
            {
                Viewpoint oCopy = oCurrV.CreateCopy();

                oCopy.Rotation = Multiply(oCurrV.Rotation, delta);

                oDoc.CurrentViewpoint.CopyFrom(oCopy);
            }
            catch (Exception ex)
            {

            }
 
            oTran.Commit();

        }

        public static Rotation3D Multiply(Rotation3D r1, Rotation3D r2)
        {
            Rotation3D res = new Rotation3D(r2.D * r1.A + r2.A * r1.D + r2.B * r1.C - r2.C * r1.B,
                                            r2.D * r1.B + r2.B * r1.D + r2.C * r1.A - r2.A * r1.C,
                                            r2.D * r1.C + r2.C * r1.D + r2.A * r1.B - r2.B * r1.A,
                                            r2.D * r1.D - r2.A * r1.A - r2.B * r1.B - r2.C * r1.C);

            return res;
        }

        private void buttonRotateCW_Click(object sender, EventArgs e)
        {
            RotateCamera(double.Parse(textRotStep.Text));
        }

        private void buttonRotateACW_Click(object sender, EventArgs e)
        {
            RotateCamera(-double.Parse(textRotStep.Text));
        }

    
    }
}
 